import React, { useEffect, useRef, useState } from 'react';
import Container from 'react-bootstrap/Container';

export type Props = {
    title: string,
    icon?: string,
    tags: string[],
    style?: 'primary' | 'light'
}

const focusChangedEventName = "focus-changed"

function useFocusedElement(elementKey: string) {
    const [isFocusedElement, setFocusedElement] = useState(false);
    const focusedClass = isFocusedElement ? 'focused-element' : ''
    const elementRef = useRef<HTMLSpanElement>(null)

    useEffect(() => {
        function getFocusedState() {
            const params = new URLSearchParams(window.location.search)
            const focusedElement = params.get('focus')
            const focused: boolean = focusedElement && focusedElement == elementKey || false
            setFocusedElement(focused)
        }

        getFocusedState()
        addEventListener(focusChangedEventName, getFocusedState)
        return () => {
            removeEventListener(focusChangedEventName, getFocusedState)
        }
    }, [elementKey, setFocusedElement])

    isFocusedElement && elementRef.current?.scrollIntoView()

    return {isFocusedElement, focusedClass, elementRef}
}

function setFocusedElement(elementKey: string) {
    const url = new URL(window.location.href);
    url.searchParams.set('focus', elementKey);
    const focusChangeEvent = new Event(focusChangedEventName);
    history.pushState({}, "", url);
    window.dispatchEvent(focusChangeEvent)
}

function Tag(props: {text: string}) {
    const tagKey = 'tag_' + props.text;
    const {focusedClass, elementRef} = useFocusedElement(tagKey)
    return (
        <span ref={elementRef}
            onClick={() => setFocusedElement(tagKey)}
            className={`badge text-bg-light ${focusedClass}`}>{props.text}</span>
    )
}

export default function TagCloud(props: Props) {
    const containerClasses = ['tag-cloud', 'cloud-' + (props.style || 'standard')]
    return (
        <Container className={containerClasses.join(' ')}>
            <h4>{props.icon && (<i className={'bi-' + props.icon}> </i>)}{props.title}</h4>
            <Container className='tag-badges'>
                {props.tags.map((tag: string) => (<Tag key={tag} text={tag} />) )}
            </Container>
        </Container>
    )
}
